@@ -28,7 +28,7 @@ from keystoneauth1 import exceptions
 from keystoneauth1.identity.v3 import federation
 from keystoneauth1 import session as ks_session
 
-_logger = utils.get_logger(__name__)
+LOG = utils.get_logger(__name__)
 
 __all__ = (
     'OidcAuthorizationCode',
@@ -58,20 +58,20 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
         identity_provider: str,
         protocol: str,
         client_id: str,
-        client_secret: ty.Optional[str],
+        client_secret: str | None,
         access_token_type: str,
         scope: str,
-        access_token_endpoint: ty.Optional[str],
-        discovery_endpoint: ty.Optional[str],
+        access_token_endpoint: str | None,
+        discovery_endpoint: str | None,
         *,
-        trust_id: ty.Optional[str],
-        system_scope: ty.Optional[str],
-        domain_id: ty.Optional[str],
-        domain_name: ty.Optional[str],
-        project_id: ty.Optional[str],
-        project_name: ty.Optional[str],
-        project_domain_id: ty.Optional[str],
-        project_domain_name: ty.Optional[str],
+        trust_id: str | None,
+        system_scope: str | None,
+        domain_id: str | None,
+        domain_name: str | None,
+        project_id: str | None,
+        project_name: str | None,
+        project_domain_id: str | None,
+        project_domain_name: str | None,
         reauthenticate: bool,
         include_catalog: bool,
     ):
@@ -172,14 +172,16 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
                     self.discovery_endpoint, authenticated=False
                 )
             except exceptions.HttpError:
-                _logger.error(
-                    f"Cannot fetch discovery document {self.discovery_endpoint}"
+                LOG.error(
+                    "Cannot fetch discovery document %s",
+                    self.discovery_endpoint,
                 )
                 raise
 
             try:
                 self._discovery_document = resp.json()
-            except Exception:
+            except Exception:  # noqa: S110
+                # we handle this below
                 pass
 
             if not self._discovery_document:
@@ -211,9 +213,7 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
             raise exceptions.OidcAccessTokenEndpointNotFound()
         return ty.cast(str, endpoint)
 
-    def _sanitize(
-        self, data: dict[str, ty.Optional[str]]
-    ) -> dict[str, ty.Optional[str]]:
+    def _sanitize(self, data: dict[str, str | None]) -> dict[str, str | None]:
         sanitized = copy.deepcopy(data)
         for key in sanitized:
             if any(s in key for s in SENSITIVE_KEYS):
@@ -221,7 +221,7 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
         return sanitized
 
     def _get_access_token(
-        self, session: ks_session.Session, payload: dict[str, ty.Optional[str]]
+        self, session: ks_session.Session, payload: dict[str, str | None]
     ) -> str:
         """Exchange a variety of user supplied values for an access token.
 
@@ -246,9 +246,9 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
             payload.setdefault('client_id', self.client_id)
         access_token_endpoint = self._get_access_token_endpoint(session)
 
-        if _logger.isEnabledFor(logging.DEBUG):
+        if LOG.isEnabledFor(logging.DEBUG):
             sanitized_payload = self._sanitize(payload)
-            _logger.debug(
+            LOG.debug(
                 "Making OpenID-Connect authentication request to %s with "
                 "data %s",
                 access_token_endpoint,
@@ -263,9 +263,9 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
             authenticated=False,
         )
         response = op_response.json()
-        if _logger.isEnabledFor(logging.DEBUG):
+        if LOG.isEnabledFor(logging.DEBUG):
             sanitized_response = self._sanitize(response)
-            _logger.debug(
+            LOG.debug(
                 "OpenID-Connect authentication response from %s is %s",
                 access_token_endpoint,
                 sanitized_response,
@@ -328,7 +328,7 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
         # First of all, check if the grant type is supported
         discovery = self._get_discovery_document(session)
         grant_types = ty.cast(
-            ty.Optional[list[str]], discovery.get("grant_types_supported")
+            list[str] | None, discovery.get("grant_types_supported")
         )
         if (
             grant_types
@@ -354,7 +354,7 @@ class _OidcBase(federation.FederationBaseAuth, metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def get_payload(
         self, session: ks_session.Session
-    ) -> dict[str, ty.Optional[str]]:
+    ) -> dict[str, str | None]:
         """Get the plugin specific payload for obtainin an access token.
 
         OpenID Connect supports different grant types. This method should
@@ -383,22 +383,22 @@ class OidcPassword(_OidcBase):
         protocol: str,
         client_id: str,
         client_secret: str,
-        access_token_type: str = 'access_token',  # nosec B107
+        access_token_type: str = 'access_token',  # noqa: S107
         scope: str = 'openid profile',
-        access_token_endpoint: ty.Optional[str] = None,
-        discovery_endpoint: ty.Optional[str] = None,
-        username: ty.Optional[str] = None,
-        password: ty.Optional[str] = None,
-        idp_otp_key: ty.Optional[str] = None,
+        access_token_endpoint: str | None = None,
+        discovery_endpoint: str | None = None,
+        username: str | None = None,
+        password: str | None = None,
+        idp_otp_key: str | None = None,
         *,
-        trust_id: ty.Optional[str] = None,
-        system_scope: ty.Optional[str] = None,
-        domain_id: ty.Optional[str] = None,
-        domain_name: ty.Optional[str] = None,
-        project_id: ty.Optional[str] = None,
-        project_name: ty.Optional[str] = None,
-        project_domain_id: ty.Optional[str] = None,
-        project_domain_name: ty.Optional[str] = None,
+        trust_id: str | None = None,
+        system_scope: str | None = None,
+        domain_id: str | None = None,
+        domain_name: str | None = None,
+        project_id: str | None = None,
+        project_name: str | None = None,
+        project_domain_id: str | None = None,
+        project_domain_name: str | None = None,
         reauthenticate: bool = True,
         include_catalog: bool = True,
     ):
@@ -437,7 +437,7 @@ class OidcPassword(_OidcBase):
 
     def get_payload(
         self, session: ks_session.Session
-    ) -> dict[str, ty.Optional[str]]:
+    ) -> dict[str, str | None]:
         """Get an authorization grant for the "password" grant type.
 
         :param session: a session object to send out HTTP requests.
@@ -450,7 +450,6 @@ class OidcPassword(_OidcBase):
             'username': self.username,
             'password': self.password,
             'scope': self.scope,
-            'client_id': self.client_id,
         }
 
         self.manage_otp_from_session_or_request_to_the_user(payload, session)
@@ -458,7 +457,7 @@ class OidcPassword(_OidcBase):
         return payload
 
     def manage_otp_from_session_or_request_to_the_user(
-        self, payload: dict[str, ty.Optional[str]], session: ks_session.Session
+        self, payload: dict[str, str | None], session: ks_session.Session
     ) -> None:
         """Get the OTP code from the session or else request to the user.
 
@@ -503,19 +502,19 @@ class OidcClientCredentials(_OidcBase):
         protocol: str,
         client_id: str,
         client_secret: str,
-        access_token_type: str = 'access_token',  # nosec B107
+        access_token_type: str = 'access_token',  # noqa: S107
         scope: str = 'openid profile',
-        access_token_endpoint: ty.Optional[str] = None,
-        discovery_endpoint: ty.Optional[str] = None,
+        access_token_endpoint: str | None = None,
+        discovery_endpoint: str | None = None,
         *,
-        trust_id: ty.Optional[str] = None,
-        system_scope: ty.Optional[str] = None,
-        domain_id: ty.Optional[str] = None,
-        domain_name: ty.Optional[str] = None,
-        project_id: ty.Optional[str] = None,
-        project_name: ty.Optional[str] = None,
-        project_domain_id: ty.Optional[str] = None,
-        project_domain_name: ty.Optional[str] = None,
+        trust_id: str | None = None,
+        system_scope: str | None = None,
+        domain_id: str | None = None,
+        domain_name: str | None = None,
+        project_id: str | None = None,
+        project_name: str | None = None,
+        project_domain_id: str | None = None,
+        project_domain_name: str | None = None,
         reauthenticate: bool = True,
         include_catalog: bool = True,
     ):
@@ -551,7 +550,7 @@ class OidcClientCredentials(_OidcBase):
 
     def get_payload(
         self, session: ks_session.Session
-    ) -> dict[str, ty.Optional[str]]:
+    ) -> dict[str, str | None]:
         """Get an authorization grant for the client credentials grant type.
 
         :param session: a session object to send out HTTP requests.
@@ -560,7 +559,7 @@ class OidcClientCredentials(_OidcBase):
         :returns: a python dictionary containing the payload to be exchanged
         :rtype: dict
         """
-        payload: dict[str, ty.Optional[str]] = {'scope': self.scope}
+        payload: dict[str, str | None] = {'scope': self.scope}
         return payload
 
 
@@ -576,23 +575,23 @@ class OidcAuthorizationCode(_OidcBase):
         protocol: str,
         client_id: str,
         client_secret: str,
-        access_token_type: str = 'access_token',  # nosec B107
+        access_token_type: str = 'access_token',  # noqa: S107
         scope: str = 'openid profile',
-        access_token_endpoint: ty.Optional[str] = None,
-        discovery_endpoint: ty.Optional[str] = None,
-        code: ty.Optional[str] = None,
+        access_token_endpoint: str | None = None,
+        discovery_endpoint: str | None = None,
+        code: str | None = None,
         *,
-        trust_id: ty.Optional[str] = None,
-        system_scope: ty.Optional[str] = None,
-        domain_id: ty.Optional[str] = None,
-        domain_name: ty.Optional[str] = None,
-        project_id: ty.Optional[str] = None,
-        project_name: ty.Optional[str] = None,
-        project_domain_id: ty.Optional[str] = None,
-        project_domain_name: ty.Optional[str] = None,
+        trust_id: str | None = None,
+        system_scope: str | None = None,
+        domain_id: str | None = None,
+        domain_name: str | None = None,
+        project_id: str | None = None,
+        project_name: str | None = None,
+        project_domain_id: str | None = None,
+        project_domain_name: str | None = None,
         reauthenticate: bool = True,
         include_catalog: bool = True,
-        redirect_uri: ty.Optional[str] = None,
+        redirect_uri: str | None = None,
     ):
         """The OpenID Authorization Code plugin expects the following.
 
@@ -629,7 +628,7 @@ class OidcAuthorizationCode(_OidcBase):
 
     def get_payload(
         self, session: ks_session.Session
-    ) -> dict[str, ty.Optional[str]]:
+    ) -> dict[str, str | None]:
         """Get an authorization grant for the "authorization_code" grant type.
 
         :param session: a session object to send out HTTP requests.
@@ -653,20 +652,20 @@ class OidcAccessToken(_OidcBase):
         protocol: str,
         # client_id and client_id intentionally omitted since they don't make
         # sense with an access token
-        access_token_type: str = 'access_token',  # nosec B107
+        access_token_type: str = 'access_token',  # noqa: S107
         scope: str = 'openid profile',
-        access_token_endpoint: ty.Optional[str] = None,
-        discovery_endpoint: ty.Optional[str] = None,
-        access_token: ty.Optional[str] = None,
+        access_token_endpoint: str | None = None,
+        discovery_endpoint: str | None = None,
+        access_token: str | None = None,
         *,
-        trust_id: ty.Optional[str] = None,
-        system_scope: ty.Optional[str] = None,
-        domain_id: ty.Optional[str] = None,
-        domain_name: ty.Optional[str] = None,
-        project_id: ty.Optional[str] = None,
-        project_name: ty.Optional[str] = None,
-        project_domain_id: ty.Optional[str] = None,
-        project_domain_name: ty.Optional[str] = None,
+        trust_id: str | None = None,
+        system_scope: str | None = None,
+        domain_id: str | None = None,
+        domain_name: str | None = None,
+        project_id: str | None = None,
+        project_name: str | None = None,
+        project_domain_id: str | None = None,
+        project_domain_name: str | None = None,
         reauthenticate: bool = True,
         include_catalog: bool = True,
     ):
@@ -715,7 +714,7 @@ class OidcAccessToken(_OidcBase):
 
     def get_payload(
         self, session: ks_session.Session
-    ) -> dict[str, ty.Optional[str]]:
+    ) -> dict[str, str | None]:
         """OidcAccessToken does not require a payload."""  # noqa: D403
         return {}
 
@@ -755,23 +754,23 @@ class OidcDeviceAuthorization(_OidcBase):
         identity_provider: str,
         protocol: str,
         client_id: str,
-        client_secret: ty.Optional[str] = None,
-        # access_token_type intentionally skipped
+        client_secret: str | None = None,
+        access_token_type: str = "access_token",  # noqa: S107
         scope: str = 'openid profile',
-        access_token_endpoint: ty.Optional[str] = None,
-        discovery_endpoint: ty.Optional[str] = None,
-        device_authorization_endpoint: ty.Optional[str] = None,
-        code_challenge: ty.Optional[str] = None,
-        code_challenge_method: ty.Optional[str] = None,
+        access_token_endpoint: str | None = None,
+        discovery_endpoint: str | None = None,
+        device_authorization_endpoint: str | None = None,
+        code_challenge: str | None = None,
+        code_challenge_method: str | None = None,
         *,
-        trust_id: ty.Optional[str] = None,
-        system_scope: ty.Optional[str] = None,
-        domain_id: ty.Optional[str] = None,
-        domain_name: ty.Optional[str] = None,
-        project_id: ty.Optional[str] = None,
-        project_name: ty.Optional[str] = None,
-        project_domain_id: ty.Optional[str] = None,
-        project_domain_name: ty.Optional[str] = None,
+        trust_id: str | None = None,
+        system_scope: str | None = None,
+        domain_id: str | None = None,
+        domain_name: str | None = None,
+        project_id: str | None = None,
+        project_name: str | None = None,
+        project_domain_id: str | None = None,
+        project_domain_name: str | None = None,
         reauthenticate: bool = True,
         include_catalog: bool = True,
     ):
@@ -788,8 +787,6 @@ class OidcDeviceAuthorization(_OidcBase):
         :param code_challenge_method: PKCE Challenge Method (RFC 7636).
         :type code_challenge_method: string
         """
-        # RFC 8628 only allows to retrieve an access_token
-        self.access_token_type = 'access_token'  # nosec B105
         self.device_authorization_endpoint = device_authorization_endpoint
         self.code_challenge_method = code_challenge_method
 
@@ -799,7 +796,7 @@ class OidcDeviceAuthorization(_OidcBase):
             protocol=protocol,
             client_id=client_id,
             client_secret=client_secret,
-            access_token_type=self.access_token_type,
+            access_token_type=access_token_type,
             scope=scope,
             access_token_endpoint=access_token_endpoint,
             discovery_endpoint=discovery_endpoint,
@@ -817,7 +814,7 @@ class OidcDeviceAuthorization(_OidcBase):
 
     def _get_device_authorization_endpoint(
         self, session: ks_session.Session
-    ) -> ty.Optional[str]:
+    ) -> str | None:
         """Get the endpoint for the OAuth 2.0 Device Authorization flow.
 
         This method will return the correct device authorization endpoint to
@@ -850,7 +847,7 @@ class OidcDeviceAuthorization(_OidcBase):
         code_verifier = _rand_b64.rstrip('=')  # strip padding as RFC says
         return code_verifier
 
-    def _generate_pkce_challenge(self) -> ty.Optional[str]:
+    def _generate_pkce_challenge(self) -> str | None:
         """Generate PKCE challenge string as defined in RFC 7636."""
         if self.code_challenge_method not in ('plain', 'S256'):
             raise exceptions.OidcInvalidCodeChallengeMethod()
@@ -867,7 +864,7 @@ class OidcDeviceAuthorization(_OidcBase):
 
     def get_payload(
         self, session: ks_session.Session
-    ) -> dict[str, ty.Optional[str]]:
+    ) -> dict[str, str | None]:
         """Get an authorization grant for the "device_code" grant type.
 
         :param session: a session object to send out HTTP requests.
@@ -876,7 +873,7 @@ class OidcDeviceAuthorization(_OidcBase):
         :returns: a python dictionary containing the payload to be exchanged
         :rtype: dict
         """
-        payload: dict[str, ty.Optional[str]]
+        payload: dict[str, str | None]
         device_authz_endpoint = self._get_device_authorization_endpoint(
             session
         )
@@ -886,10 +883,13 @@ class OidcDeviceAuthorization(_OidcBase):
 
         if self.client_secret:
             client_auth = (self.client_id, self.client_secret)
-            payload = {}
         else:
             client_auth = None
-            payload = {'client_id': self.client_id}
+
+        # rfc8628 does not require client_id when a client_secret is provided,
+        # but Microsoft EntraID does; these should be ignored by other IDPs
+        # when a client secret is provided.
+        payload = {'client_id': self.client_id, 'scope': self.scope}
 
         if self.code_challenge_method:
             self.code_challenge = self._generate_pkce_challenge()
@@ -899,9 +899,9 @@ class OidcDeviceAuthorization(_OidcBase):
             payload.setdefault('code_challenge', self.code_challenge)
         encoded_payload = urlparse.urlencode(payload)
 
-        if _logger.isEnabledFor(logging.DEBUG):
+        if LOG.isEnabledFor(logging.DEBUG):
             sanitized_payload = self._sanitize(payload)
-            _logger.debug(
+            LOG.debug(
                 "Making OpenID-Connect authentication request to %s with "
                 "data %s",
                 device_authz_endpoint,
@@ -915,9 +915,9 @@ class OidcDeviceAuthorization(_OidcBase):
             log=False,
             authenticated=False,
         )
-        if _logger.isEnabledFor(logging.DEBUG):
+        if LOG.isEnabledFor(logging.DEBUG):
             sanitized_response = self._sanitize(op_response.json())
-            _logger.debug(
+            LOG.debug(
                 "OpenID-Connect authentication response from %s is %s",
                 device_authz_endpoint,
                 sanitized_response,
@@ -929,9 +929,9 @@ class OidcDeviceAuthorization(_OidcBase):
         self.interval = int(op_response.json()["interval"])
         self.user_code = op_response.json()["user_code"]
         self.verification_uri = op_response.json()["verification_uri"]
-        self.verification_uri_complete = op_response.json()[
+        self.verification_uri_complete = op_response.json().get(
             "verification_uri_complete"
-        ]
+        )
 
         payload = {'device_code': self.device_code}
         if self.code_challenge_method:
@@ -939,7 +939,7 @@ class OidcDeviceAuthorization(_OidcBase):
         return payload
 
     def _get_access_token(
-        self, session: ks_session.Session, payload: dict[str, ty.Optional[str]]
+        self, session: ks_session.Session, payload: dict[str, str | None]
     ) -> str:
         """Poll token endpoint for an access token.
 
@@ -954,25 +954,38 @@ class OidcDeviceAuthorization(_OidcBase):
                     'device_code': self.device_code,
                 }
         :type payload: dict
-        """
-        _logger.warning(
-            f"To authenticate please go to: {self.verification_uri_complete}"
-        )
+        """  # noqa: E501
+        # verification_uri_complete is optional and not implemented by EntraID
+        if self.verification_uri_complete:
+            LOG.warning(
+                "To authenticate please go to: %s",
+                self.verification_uri_complete,
+            )
+        else:
+            LOG.warning(
+                "To authenticate please go to %s and enter the code %s",
+                self.verification_uri,
+                self.user_code,
+            )
 
         if self.client_secret:
             client_auth = (self.client_id, self.client_secret)
         else:
             client_auth = None
-            payload.setdefault('client_id', self.client_id)
+
+        # rfc8628 does not require client_id when a client_secret is provided,
+        # but Microsoft EntraID does; this should be ignored by other IDPs
+        # when a client secret is provided.
+        payload.setdefault('client_id', self.client_id)
 
         access_token_endpoint = self._get_access_token_endpoint(session)
         encoded_payload = urlparse.urlencode(payload)
 
         while time.time() < self.timeout:
             try:
-                if _logger.isEnabledFor(logging.DEBUG):
+                if LOG.isEnabledFor(logging.DEBUG):
                     sanitized_payload = self._sanitize(payload)
-                    _logger.debug(
+                    LOG.debug(
                         "Making OpenID-Connect authentication request to %s "
                         "with data %s",
                         access_token_endpoint,
@@ -986,9 +999,9 @@ class OidcDeviceAuthorization(_OidcBase):
                     log=False,
                     authenticated=False,
                 )
-                if _logger.isEnabledFor(logging.DEBUG):
+                if LOG.isEnabledFor(logging.DEBUG):
                     sanitized_response = self._sanitize(op_response.json())
-                    _logger.debug(
+                    LOG.debug(
                         "OpenID-Connect authentication response from %s is %s",
                         access_token_endpoint,
                         sanitized_response,
